package main

import (
	"bufio"
	"fmt"
	"os"
	"strings"

	"github.com/manifoldco/promptui"
	"inet.af/netaddr"
)

var (
	cufrd *bufio.Reader
	xerox *bufio.Scanner
)

func init() {
	dbfile, err := os.OpenFile("ripe_cleaned.db", os.O_RDWR|os.O_CREATE, 0600)
	if err != nil {
		println(err.Error())
		os.Exit(1)
	}
	cufrd = bufio.NewReader(dbfile)
	xerox = bufio.NewScanner(cufrd)
}

type Entry struct {
	Name         string             `json:"origin_asn,omitempty"`
	NetName      string             `json:"netname,omitempty"`
	RoutePrefix  []netaddr.IPPrefix `json:"route,omitempty"`
	IPRange      []netaddr.IPRange  `json:"iprange,omitempty"`
	Description  []string           `json:"description,omitempty"`
	MaintainedBy []string           `json:"maintained_by,omitempty"`
	MemberOf     []string           `json:"member_of,omitempty"`
	Notify       []string           `json:"notify,omitempty"`
	Source       string             `json:"source,omitempty"`
}

var (
	asnMap    = make(map[string]*Entry)
	prefixMap = make(map[string][]*Entry)
)

var (
	tmpMap     = make(map[int]*Entry)
	count      = 0
	innerCount = 0
)

var funcmap = promptui.FuncMap

func init() {
	funcmap["getentry"] = func(s string) *Entry {
		panic(s)
		return asnMap[s]
	}
}

func processLine(line string) {
	if len(strings.TrimSpace(line)) == 0 || count == 0 {
		if tmpMap[count] != nil && tmpMap[count].Name != "" {
			asnMap[tmpMap[count].Name] = tmpMap[count]
		}
		if tmpMap[count] != nil && tmpMap[count].RoutePrefix != nil {
			for _, prefix := range tmpMap[count].RoutePrefix {
				prefixMap[prefix.String()] = append(
					prefixMap[prefix.String()],
					tmpMap[count],
				)
			}
		}
		count++
		tmpMap[count] = &Entry{}
		innerCount = 0
		if count > 1 {
			return
		}
	}
	innerCount++
	lineFields := strings.Fields(line)
	switch {
	case len(lineFields) < 2:
		return
	case innerCount == 1 &&
		!strings.HasPrefix(line, "route:") &&
		!strings.HasPrefix(line, "inetnum:"):
		return
	case innerCount == 1 && strings.HasPrefix(line, "route:"):
		route, err := netaddr.ParseIPPrefix(lineFields[1])
		if err != nil {
			println(err.Error())
			return
		}
		tmpMap[count].RoutePrefix = append(tmpMap[count].RoutePrefix, route)
	case innerCount == 1 && strings.HasPrefix(line, "inetnum:"):
		ipr, err := netaddr.ParseIPRange(strings.Join(lineFields[1:], ""))
		if err == nil {
			tmpMap[count].IPRange = append(tmpMap[count].IPRange, ipr)
		}
	case strings.HasPrefix(line, "origin:"):
		tmpMap[count].Name = lineFields[1]
	case strings.HasPrefix(line, "descr:"):
		tmpMap[count].Description = append(tmpMap[count].Description, strings.Join(lineFields[1:], " "))
	case strings.HasPrefix(line, "netname:"):
		tmpMap[count].NetName = lineFields[1]
	case strings.HasPrefix(line, "mnt-by:"),
		strings.HasPrefix(line, "mnt-lower"),
		strings.HasPrefix(line, "mnt-routes"),
		strings.HasPrefix(line, "mnt-domains"):
		tmpMap[count].MaintainedBy = append(tmpMap[count].MaintainedBy, lineFields[1])
	case strings.HasPrefix(line, "source:"):
		tmpMap[count].Source = lineFields[1]
	case strings.HasPrefix(line, "notify:"):
		tmpMap[count].Notify = append(tmpMap[count].Notify, lineFields[1])
	case strings.HasPrefix(line, "member-of:"):
		tmpMap[count].MemberOf = append(tmpMap[count].MemberOf, lineFields[1])
	}
}

func main() {
	for xerox.Scan() {
		processLine(xerox.Text())
	}
	fmt.Println("Total entries:", len(asnMap))

	var asnNames []string
	for _, entry := range asnMap {
		if entry.Name != "" {
			asnNames = append(asnNames, entry.Name)
		}
	}

	// spew.Dump(asnNames)

	search := func(input string, index int) bool {
		trg := asnNames[index]
		aut, ok := asnMap[trg]
		if !ok {
			return false
		}
		input = strings.ToLower(input)
		name := strings.ToLower(aut.Name)
		if strings.Contains(name, input) {
			return true
		}
		if len(aut.NetName) > 0 {
			netname := strings.ToLower(aut.NetName)
			if strings.Contains(netname, input) {
				return true
			}
		}
		if len(aut.Description) > 0 {
			for _, desc := range aut.Description {
				if strings.Contains(strings.ToLower(desc), input) {
					return true
				}
			}
		}
		return false
	}

	selector := promptui.Select{
		Label:     "Known ASNs",
		Items:     asnNames,
		Size:      25,
		CursorPos: 0,
		Templates: &promptui.SelectTemplates{
			FuncMap:  funcmap,
			Label:    "{{ . | bold }}",
			Active:   "{{ $entry := (getentry .Name ) }} > {{ $entry.OriginASN | cyan | bold }} {{ $entry.Route | green }}",
			Inactive: "{{ $entry := (getentry .Name ) }} {{ $entry.OriginASN | faint }} {{ $entry.Route | faint }}",
			Selected: "{{ $entry := (getentry .Name ) }} > {{ $entry.OriginASN | cyan | bold }} {{ $entry.Route | green }}",
			Details: `
------- Autonomous System -------- {{ with $entry := (getentry .Name) }} 
{{ "  ASN:           " | faint }} {{ $entry.OriginASN }}
{{ "  Route:         " | faint }} {{ printf "%s" $entry.Route | green }}
{{ "  Description:   " | faint }} {{ printf "%+s" $entry.Description | cyan }}
{{ "  Maintained By: " | faint }} {{ printf "%+s" $entry.MaintainedBy }}
{{ "  Member Of:     " | faint }} {{ printf "%s" $entry.MemberOf }}
{{ "  Notify:        " | faint }} {{ printf "%s" $entry.Notify }}
{{ "  Source:        " | faint }} {{ printf "%s" $entry.Source }}{{end}}`,
		},
		Pointer: func(to []rune) []rune {
			return []rune(">")
		},
		Stdin:    os.Stdin,
		Stdout:   os.Stdout,
		Searcher: search,
	}

	_, _, err := selector.Run()
	if err != nil {
		println(err.Error())
		os.Exit(1)
	}
}
